/* /////////////////////////////////////////////////////////////////////////////
 * File:        example_c_7.c
 *
 * Purpose:     C example program for the recls core library. Demonstrates:
 *
 *                - stat() of current directory (via Recls_Stat())
 *                - combining paths (via Recls_CombinePaths())
 *                - elicitation of entry properties via structure members
 *                - handling of errors and reporting of error information
 *
 * Created:     17th June 2006
 * Updated:     18th June 2006
 *
 * www:         http://www.recls.org/
 *
 * License:     Copyright (c) 2006, Synesis Software Pty Ltd.
 *              All rights reserved.
 *
 *              (Licensed under the Synesis Software Open License)
 *
 *              This source code is placed into the public domain 2006
 *              by Synesis Software Pty Ltd. There are no restrictions
 *              whatsoever to your use of the software.
 *
 * ////////////////////////////////////////////////////////////////////// */

/* recls Header Files */
#include <recls/recls.h>

/* Standard C Library Files */
#include <stdio.h>      /* for printf() / fprintf()         */
#include <stdlib.h>     /* for EXIT_SUCCESS / EXIT_FAILURE  */
#include <string.h>

/* /////////////////////////////////////////////////////////////////////////
 * Macros and definitions
 */

#ifndef RECLS_NUM_ELEMENTS
# define RECLS_NUM_ELEMENTS(x)          (sizeof(x) / sizeof((x)[0]))
#endif /* !RECLS_NUM_ELEMENTS */

/* ////////////////////////////////////////////////////////////////////// */

int main()
{
    /* stat() the current directory */
    recls_info_t    current;
    recls_rc_t      rc  =   Recls_Stat(".", RECLS_F_DIRECTORIES | RECLS_F_DIRECTORY_PARTS, &current);

    if(RECLS_FAILED(rc))
    {
        /* The search failed. Display the error string. */
        char    err[1001];
        size_t  n   =   Recls_GetErrorString(rc, &err[0], sizeof(err) - 1);

        err[n] = '\0';

        fprintf(stderr, "stat of current directory failed: %s\n", err);

        return EXIT_FAILURE;
    }
    else
    {
        const char  path2[] =   "abc/def/";
        char        combinedPath[1001];
        size_t      cch     =   Recls_CombinePaths(current->path.begin, path2, &combinedPath[0], RECLS_NUM_ELEMENTS(combinedPath));

        /* full path */
        printf("%s combined with %s yields %.*s\n", current->path.begin, path2, (int)cch, combinedPath);

        /* Close the current entry. */
        Recls_CloseDetails(current);

        return EXIT_SUCCESS;
    }
}

/* ////////////////////////////////////////////////////////////////////// */
