////////////////////////////////////////////////////////////////////////////////
// FileSearch.cpp : Implementation of FileSearch
//
// Created: 18th March 2006
// Updated: 14th June 2006

#include "stdafx.h"
#include "recls.com.h"
#include "FileSearch.h"
#include "SearchCollection.h"

#include <stlsoft/memory/auto_buffer.hpp>

#include <comstl/string/bstr_functions.h>

////////////////////////////////////////////////////////////////////////////////
// IFileSearch

FileSearch::FileSearch()
{}

FileSearch::~FileSearch() stlsoft_throw_0()
{}

////////////////////////////////////////////////////////////////////////////////
// Implementation

HRESULT FileSearch::CombinePaths_(recls_char_t const *path1, recls_char_t const *path2, BSTR &result)
{
    ATLASSERT(NULL == result);

    stlsoft::auto_buffer<recls_char_t>  buffer(Recls_CombinePaths(path1, path2, NULL, 0));

    Recls_CombinePaths(path1, path2, &buffer[0], buffer.size());

    result = comstl::bstr_create(buffer.data(), buffer.size());

    return (NULL == result) ? E_OUTOFMEMORY : S_OK;
}

HRESULT FileSearch::DeriveRelativePath_(recls_char_t const *origin, recls_char_t const *target, BSTR &result)
{
    ATLASSERT(NULL == result);

    stlsoft::auto_buffer<recls_char_t>  buffer(Recls_DeriveRelativePath(origin, target, NULL, 0));

    Recls_DeriveRelativePath(origin, target, &buffer[0], buffer.size());

    result = comstl::bstr_create(buffer.data(), buffer.size());

    return (NULL == result) ? E_OUTOFMEMORY : S_OK;
}

HRESULT FileSearch::SqueezePath_(recls_char_t const *path, size_t width, BSTR &result)
{
    ATLASSERT(NULL == result);

    stlsoft::auto_buffer<recls_char_t>  buffer(width);

    Recls_SqueezePath(path, &buffer[0], buffer.size());

    result = comstl::bstr_create(buffer.data(), buffer.size());

    return (NULL == result) ? E_OUTOFMEMORY : S_OK;
}

////////////////////////////////////////////////////////////////////////////////
// IFileSearch

STDMETHODIMP FileSearch::Search(BSTR searchRoot, BSTR pattern, long flags, IUnknown **ppenum)
{
    HRESULT hr;

    if(NULL == ppenum)
    {
        hr = E_POINTER;
    }
    else
    {
        *ppenum = NULL;

        hrecls_t    hSrch;
        recls_rc_t  rc  =   call_Search(searchRoot
                                    ,   pattern
                                    ,   flags
                                    ,   NULL
                                    ,   &hSrch);

        if( RECLS_FAILED(rc) && 
            rc != RECLS_RC_NO_MORE_DATA)
        {
            set_error_info_from_recls_rc_t(rc);

            hr = E_FAIL;
        }
        else
        {
            typedef CComObject<SearchCollection>    Search_t;

            Search_t    *pSrch = new Search_t();

            if(NULL == pSrch)
            {
                Recls_SearchClose(hSrch);

                hr = E_OUTOFMEMORY;
            }
            else
            {
                pSrch->AddRef();

                SearchInfo  info;

                if(RECLS_RC_NO_MORE_DATA == rc)
                {
                    info.bEmpty     =   true;
                }
                else
                {
                    info.hSrch      =   hSrch;
                    info.flags      =   flags;
                    info.pattern    =   pattern;
                    info.searchRoot =   searchRoot;
                }

                pSrch->SetVoid(&info);
                ATLASSERT(NULL == info.hSrch);

                *ppenum = pSrch;

                hr = S_OK;
            }
        }
    }

    return hr;
}

// IFileSearch2
STDMETHODIMP FileSearch::SearchProgress(/*[in]*/ BSTR searchRoot, /*[in]*/ BSTR pattern, /*[in]*/ long flags, /*[in]*/ LPUNKNOWN punkProgress, /*[out, retval]*/ IUnknown **ppenum)
{
    HRESULT hr;

    if(NULL == ppenum)
    {
        hr = E_POINTER;
    }
    else
    {
        *ppenum = NULL;

        hrecls_t    hSrch;
        recls_rc_t  rc  =   call_Search(searchRoot
                                    ,   pattern
                                    ,   flags
                                    ,   punkProgress
                                    ,   &hSrch);

        if( RECLS_FAILED(rc) && 
            rc != RECLS_RC_NO_MORE_DATA)
        {
            set_error_info_from_recls_rc_t(rc);

            hr = E_FAIL;
        }
        else
        {
            typedef CComObject<SearchCollection>    Search_t;

            Search_t    *pSrch = new Search_t();

            if(NULL == pSrch)
            {
                Recls_SearchClose(hSrch);

                hr = E_OUTOFMEMORY;
            }
            else
            {
                pSrch->AddRef();

                pSrch->SetProgress(punkProgress);

                SearchInfo  info;

                if(RECLS_RC_NO_MORE_DATA == rc)
                {
                    info.bEmpty     =   true;
                }
                else
                {
                    info.hSrch      =   hSrch;
                    info.flags      =   flags;
                    info.pattern    =   pattern;
                    info.searchRoot =   searchRoot;
                }

                pSrch->SetVoid(&info);
                ATLASSERT(NULL == info.hSrch);

                *ppenum = pSrch;

                hr = S_OK;
            }
        }
    }

    return hr;
}

STDMETHODIMP FileSearch::get_Roots(/* [out, retval] */ LPUNKNOWN *pVal)
{
    HRESULT hr;

    if(NULL == pVal)
    {
        hr = E_POINTER;
    }
    else
    {
        *pVal = NULL;

        recls_root_t    roots[26];
        size_t          cRoots  =   Recls_GetRoots(roots, STLSOFT_NUM_ELEMENTS(roots));

        hr = get_Roots_(roots, cRoots, pVal);
    }

    return hr;
}

HRESULT FileSearch::get_Roots_(recls_root_t roots[], size_t cRoots, LPUNKNOWN *pVal)
{
    ATLSTL_ASSERT(NULL != pVal);

    HRESULT     hr  =   S_OK;
    OLECHAR     ole_roots[26][4];
    LPOLESTR    root_ptrs[26];

    stlsoft_ns_qual_std(copy)(&ole_roots[0], &ole_roots[stlsoft_num_elements(ole_roots)], &root_ptrs[0]);

    for(size_t i = 0; i < cRoots; ++i)
    {
#ifdef RECLS_CHAR_TYPE_IS_WCHAR
        lstrcpyW(root_ptrs[i], roots[i].name);
#else /* ? RECLS_CHAR_TYPE_IS_WCHAR */
        ::MultiByteToWideChar(0, 0, roots[i].name, 4, ole_roots[i], 4);
#endif /* RECLS_CHAR_TYPE_IS_WCHAR */
    }

    typedef CComObject< CComEnum<IEnumVARIANT, &IID_IEnumVARIANT, VARIANT, _Copy<VARIANT> > >   enumerator_t;
    typedef CComObject<atlstl::generic_automation_collection<enumerator_t> >                    collection_t;

    enumerator_t    *pe     =   new enumerator_t;

    if(NULL == pe)
    {
        hr = E_OUTOFMEMORY;
    }
    else
    {
        VARIANT     vars[26];
        VARIANT     *var_ptrs[26];

        { for(size_t i = 0; i < cRoots; ++i)
        {
            var_ptrs[i] = vars + i;
        }}

        pe->AddRef();

        std::for_each(&var_ptrs[0], &var_ptrs[0] + cRoots, ::VariantInit);

        { for(size_t i = 0; i < cRoots && SUCCEEDED(hr); ++i)
        {
            vars[i].vt      =   VT_BSTR;
            vars[i].bstrVal =   ::SysAllocString(root_ptrs[i]);

            if(NULL == vars[i].bstrVal)
            {
                hr = E_OUTOFMEMORY;
            }
        }}

        if(SUCCEEDED(hr))
        {
            hr = pe->Init(&vars[0], &vars[cRoots], NULL, AtlFlagCopy);

            collection_t    *coll   =   new collection_t;

            if(NULL == coll)
            {
                hr = E_OUTOFMEMORY;
            }
            else
            {
                coll->AddRef();

                coll->SetEnumerator(pe, true);

                if(SUCCEEDED(hr))
                {
                    hr = coll->QueryInterface(IID_IUnknown, reinterpret_cast<void**>(pVal));
                }

                coll->Release();
            }
        }

        std::for_each(&var_ptrs[0], &var_ptrs[0] + cRoots, ::VariantClear);

        pe->Release();
    }

    return hr;
}

STDMETHODIMP FileSearch::get_PathNameSeparator(BSTR *pVal)
{
    HRESULT hr;

    if(NULL == pVal)
    {
        hr = E_POINTER;
    }
    else
    {
        USES_CONVERSION;

        *pVal = ::SysAllocString(T2COLE(Recls_GetPathNameSeparator()));

        hr = (NULL == *pVal) ? E_OUTOFMEMORY : S_OK;
    }

    return hr;
}

STDMETHODIMP FileSearch::get_PathSeparator(BSTR *pVal)
{
    HRESULT hr;

    if(NULL == pVal)
    {
        hr = E_POINTER;
    }
    else
    {
        USES_CONVERSION;

        *pVal = ::SysAllocString(T2COLE(Recls_GetPathSeparator()));

        hr = (NULL == *pVal) ? E_OUTOFMEMORY : S_OK;
    }

    return hr;
}

STDMETHODIMP FileSearch::get_WildcardsAll(BSTR *pVal)
{
    HRESULT hr;

    if(NULL == pVal)
    {
        hr = E_POINTER;
    }
    else
    {
        USES_CONVERSION;

        *pVal = ::SysAllocString(T2COLE(Recls_GetWildcardsAll()));

        hr = (NULL == *pVal) ? E_OUTOFMEMORY : S_OK;
    }

    return hr;
}

////////////////////////////////////////////////////////////////////////////////
// IFileSearch3

STDMETHODIMP FileSearch::get_SelectedRoots(/*[in]*/ DWORD flags, /*[out,retval]*/ LPUNKNOWN *pVal)
{
    HRESULT hr  =   S_OK;

    if(NULL == pVal)
    {
        hr = E_POINTER;
    }
    else
    {
        *pVal = NULL;

        recls_root_t    roots[26];
        size_t          cRoots  =   Recls_GetSelectedRoots(roots, STLSOFT_NUM_ELEMENTS(roots), flags);

        hr = get_Roots_(roots, cRoots, pVal);
    }

    return hr;
}

////////////////////////////////////////////////////////////////////////////////
// IFileSearch3

STDMETHODIMP FileSearch::CombinePaths(/* [in, string] */ BSTR path1, /* [in, string] */ BSTR path2, /* [out, retval] */ BSTR *result)
{
    if(NULL == result)
    {
        return E_POINTER;
    }
    else
    {
        USES_CONVERSION;

        *result = NULL;

        return CombinePaths_(W2CRT(path1), W2CRT(path2), *result);
    }
}

STDMETHODIMP FileSearch::DeriveRelativePath(/* [in, string] */ BSTR origin, /* [in, string] */ BSTR target, /* [out, retval] */ BSTR *result)
{
    if(NULL == result)
    {
        return E_POINTER;
    }
    else
    {
        USES_CONVERSION;

        *result = NULL;

        return DeriveRelativePath_(W2CRT(origin), W2CRT(target), *result);
    }
}

STDMETHODIMP FileSearch::SqueezePath(/* [in, string] */ BSTR path, /* [in] */ long width, /* [out, retval] */ BSTR *result)
{
    if(NULL == result)
    {
        return E_POINTER;
    }
    else if(width < 0)
    {
        return E_INVALIDARG;
    }
    else
    {
        USES_CONVERSION;

        *result = NULL;

        return SqueezePath_(W2CRT(path), static_cast<size_t>(width), *result);
    }
}

////////////////////////////////////////////////////////////////////////////////
// IFtpSearch

STDMETHODIMP FileSearch::Search(/*[in]*/ BSTR host, /*[in]*/ BSTR username, /*[in]*/ BSTR password, /*[in]*/ BSTR searchRoot, /*[in]*/ BSTR pattern, /*[in]*/ long flags, /*[out, retval]*/ IUnknown **ppenum)
{
    HRESULT hr;

    if( NULL != username &&
        '\0' == *username)
    {
        username = NULL;
    }

    if( NULL != password &&
        '\0' == *password)
    {
        password = NULL;
    }

    if(NULL == ppenum)
    {
        hr = E_POINTER;
    }
    else
    {
        *ppenum = NULL;

        hrecls_t    hSrch;

#ifdef RECLS_CHAR_TYPE_IS_CHAR
        recls_rc_t  rc  =   Recls_SearchFtp(winstl::w2a(host), winstl::w2a(username), winstl::w2a(password), winstl::w2a(searchRoot), winstl::w2a(pattern), static_cast<recls_uint32_t>(flags), &hSrch);
#elif defined(RECLS_CHAR_TYPE_IS_WCHAR)
        recls_rc_t  rc  =   Recls_SearchFtp(host, username, password, searchRoot, pattern, static_cast<recls_uint32_t>(flags), &hSrch);
#endif /* RECLS_CHAR_TYPE_IS_WCHAR */

        if( RECLS_FAILED(rc) && 
            rc != RECLS_RC_NO_MORE_DATA)
        {
            set_error_info_from_recls_rc_t(rc);

            hr = E_FAIL;
        }
        else
        {
            typedef CComObject<SearchCollection>    Search_t;

            Search_t    *pSrch = new Search_t();

            if(NULL == pSrch)
            {
                Recls_SearchClose(hSrch);

                hr = E_OUTOFMEMORY;
            }
            else
            {
                pSrch->AddRef();

                SearchInfo  info;

                if(RECLS_RC_NO_MORE_DATA == rc)
                {
                    info.bEmpty     =   true;
                }
                else
                {
                    info.hSrch      =   hSrch;
                    info.flags      =   flags;
                    info.pattern    =   pattern;
                    info.searchRoot =   searchRoot;
                }

                pSrch->SetVoid(&info);
                ATLASSERT(NULL == info.hSrch);

                *ppenum = pSrch;

                hr = S_OK;
            }
        }
    }

    return hr;
}

////////////////////////////////////////////////////////////////////////////////
