/* /////////////////////////////////////////////////////////////////////////////
 * File:        recls_Entry.cpp
 *
 * Purpose:     JNI implementation file for the Entry class
 *
 * Created      21st November 2003
 * Updated:     13th June 2006
 *
 * Home:        http://recls.org/
 *
 * Copyright (c) 2003-2005, Matthew Wilson and Synesis Software
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 *
 * - Redistributions of source code must retain the above copyright notice, this
 *   list of conditions and the following disclaimer. 
 * - Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 * - Neither the name(s) of Matthew Wilson and Synesis Software nor the names of
 *   any contributors may be used to endorse or promote products derived from
 *   this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * ////////////////////////////////////////////////////////////////////////// */


#include <recls_Entry.h>

#include <recls/recls.h>
#if defined(RECLS_PLATFORM_IS_UNIX)
# include <recls_unix.h>
#elif defined(RECLS_PLATFORM_IS_WIN32)
# include <recls_win32.h>
#else
# error Platform not (yet) recognised
#endif /* platform */

#include "recls_jni_util.h"

#include <platformstl/filesystem/file_path_buffer.hpp>
#if defined(RECLS_PLATFORM_IS_WIN32)
# include <winstl/time_conversion_functions.h>
#endif /* platform */

#include <stdarg.h>

/* ////////////////////////////////////////////////////////////////////////// */

#if !defined(RECLS_NO_NAMESPACE)
using recls::recls_info_t;

using recls::Recls_CloseDetails;
# if defined(RECLS_PLATFORM_IS_WIN32)
using recls::Recls_GetDriveProperty;
# endif /* RECLS_PLATFORM_IS_WIN32 */
using recls::Recls_GetDirectoryProperty;
using recls::Recls_GetDirectoryPathProperty;
using recls::Recls_GetUNCDriveProperty;
using recls::Recls_GetFileProperty;
using recls::Recls_GetShortFileProperty;
using recls::Recls_GetFileNameProperty;
using recls::Recls_GetFileExtProperty;
using recls::Recls_GetCreationTime;
using recls::Recls_GetModificationTime;
using recls::Recls_GetLastAccessTime;
using recls::Recls_GetLastStatusChangeTime;
using recls::Recls_GetSizeProperty;
using recls::Recls_IsFileReadOnly;
using recls::Recls_IsFileDirectory;
using recls::Recls_IsFileLink;
using recls::Recls_IsFileUNC;
#endif /* !RECLS_NO_NAMESPACE */

#if defined(STLSOFT_COMPILER_IS_MSVC) && \
	_MSC_VER < 1310
# if defined(RECLS_PLATFORM_IS_UNIX)
typedef unixstl::basic_file_path_buffer<recls::recls_char_t>	file_path_buffer;
# elif defined(RECLS_PLATFORM_IS_WIN32)
typedef winstl::basic_file_path_buffer<recls::recls_char_t>		file_path_buffer;
# else /* ? platform */
#  error Platform not (yet) recognised
# endif /* platform */
#else /* ? compiler */
typedef platformstl::basic_file_path_buffer<recls::recls_char_t>	file_path_buffer;
#endif /* compiler */

/* ////////////////////////////////////////////////////////////////////////// */

jfieldID GetEntryFieldID(JNIEnv *henv, jobject obj);
recls_info_t EntryFromJObject(JNIEnv *henv, jobject obj);

jlong GetEntryTimeProperty(JNIEnv *henv, jobject obj, recls_time_t (RECLS_CALLCONV_DEFAULT *pfn)(recls_info_t));
jstring GetEntryStringProperty(JNIEnv *henv, jobject obj, size_t (RECLS_CALLCONV_DEFAULT *pfn)(recls_info_t, recls_char_t*, size_t));
jboolean GetEntryBooleanProperty(JNIEnv *henv, jobject obj, recls_bool_t (RECLS_CALLCONV_DEFAULT *pfn)(recls_info_t));

/* ////////////////////////////////////////////////////////////////////////// */

/*
 * Class:     org_recls_Entry
 * Method:    close
 * Signature: ()V
 */
JNIEXPORT void JNICALL Java_org_recls_Entry_close
  (JNIEnv *henv, jobject obj)
{
	// Get the handle from the object
	recls_info_t	hEntry	=	EntryFromJObject(henv, obj);

	if(NULL == hEntry)
	{
//		ThrowException(henv, CLASSNAME_ILLEGALSTATEX, "Entry is already closed");
	}
	else
	{
		jfieldID	fid			=	GetEntryFieldID(henv, obj);

		henv->SetIntField(obj, fid, 0);

		Recls_CloseDetails(hEntry);
	}
}

/*
 * Class:     org_recls_Entry
 * Method:    getDrive
 * Signature: ()C
 */
JNIEXPORT jchar JNICALL Java_org_recls_Entry_getDrive
  (JNIEnv *henv, jobject obj)
{
#if defined(RECLS_PLATFORM_IS_WIN32)
	recls_info_t	hEntry	=	EntryFromJObject(henv, obj);
	recls_char_t	chDrive;

	if(NULL == hEntry)
	{
		ThrowException(henv, CLASSNAME_ILLEGALSTATEX, "Entry is closed");

		chDrive = 0;
	}
	else
	{
		Recls_GetDriveProperty(hEntry, &chDrive);
	}

	return static_cast<jchar>(chDrive);
#else /* ? RECLS_PLATFORM_IS_WIN32 */

	ThrowException(henv, CLASSNAME_UNSUPPORTEDX, "The Drive property is only accessible on Win32 systems");

	return 0;
#endif /* RECLS_PLATFORM_IS_WIN32 */
}

/*
 * Class:     org_recls_Entry
 * Method:    getDirectory
 * Signature: ()Ljava/lang/String;
 */
JNIEXPORT jstring JNICALL Java_org_recls_Entry_getDirectory
  (JNIEnv *henv, jobject obj)
{
	recls_info_t	hEntry	=	EntryFromJObject(henv, obj);

	if(NULL == hEntry)
	{
		ThrowException(henv, CLASSNAME_ILLEGALSTATEX, "Entry is closed");

		return NULL;
	}
	else
	{
		file_path_buffer	buffer;
		size_t				cch = Recls_GetDirectoryProperty(hEntry, &buffer[0], buffer.size());
	
#ifdef RECLS_CHAR_TYPE_IS_CHAR
		return henv->NewStringUTF(buffer.c_str());
#elif defined(RECLS_CHAR_TYPE_IS_WCHAR
		return henv->NewString(buffer.c_str());
#else /* ?char type */
# error recls char type must be discriminated
#endif /* char type */
	} 
}

/*
 * Class:     org_recls_Entry
 * Method:    getDirectoryPath
 * Signature: ()Ljava/lang/String;
 */
JNIEXPORT jstring JNICALL Java_org_recls_Entry_getDirectoryPath
  (JNIEnv *henv, jobject obj)
{
	return GetEntryStringProperty(henv, obj, Recls_GetDirectoryPathProperty);
}

/*
 * Class:     org_recls_Entry
 * Method:    getUNCDrive
 * Signature: ()Ljava/lang/String;
 */
JNIEXPORT jstring JNICALL Java_org_recls_Entry_getUNCDrive
  (JNIEnv *henv, jobject obj)
{
	return GetEntryStringProperty(henv, obj, Recls_GetUNCDriveProperty);
}

/*
 * Class:     org_recls_Entry
 * Method:    getFile
 * Signature: ()Ljava/lang/String;
 */
JNIEXPORT jstring JNICALL Java_org_recls_Entry_getFile
  (JNIEnv *henv, jobject obj)
{
	return GetEntryStringProperty(henv, obj, Recls_GetFileProperty);
}

/*
 * Class:     org_recls_Entry
 * Method:    getShortFile
 * Signature: ()Ljava/lang/String;
 */
JNIEXPORT jstring JNICALL Java_org_recls_Entry_getShortFile
  (JNIEnv *henv, jobject obj)
{
	return GetEntryStringProperty(henv, obj, Recls_GetShortFileProperty);
}

/*
 * Class:     org_recls_Entry
 * Method:    getFileName
 * Signature: ()Ljava/lang/String;
 */
JNIEXPORT jstring JNICALL Java_org_recls_Entry_getFileName
  (JNIEnv *henv, jobject obj)
{
	return GetEntryStringProperty(henv, obj, Recls_GetFileNameProperty);
}

/*
 * Class:     org_recls_Entry
 * Method:    getFileExt
 * Signature: ()Ljava/lang/String;
 */
JNIEXPORT jstring JNICALL Java_org_recls_Entry_getFileExt
  (JNIEnv *henv, jobject obj)
{
	return GetEntryStringProperty(henv, obj, Recls_GetFileExtProperty);
}

/*
 * Class:     org_recls_Entry
 * Method:    getSize
 * Signature: ()J
 */
JNIEXPORT jlong JNICALL Java_org_recls_Entry_getSize
  (JNIEnv *henv, jobject obj)
{
	recls_info_t	hEntry	=	EntryFromJObject(henv, obj);

	if(NULL == hEntry)
	{
		ThrowException(henv, CLASSNAME_ILLEGALSTATEX, "Entry is closed");

		return NULL;
	}
	else
	{
		recls_filesize_t	size;

		Recls_GetSizeProperty(hEntry, &size);

#ifdef RECLS_PLATFORM_IS_WIN32
		return size.QuadPart;
#elif defined(RECLS_PLATFORM_IS_UNIX)
		return size;
#else /* ? operating system */
# error Platform not (yet) recognised
#endif /* operating system */
	} 
}

/*
 * Class:     org_recls_Entry
 * Method:    isReadOnly
 * Signature: ()Z
 */
JNIEXPORT jboolean JNICALL Java_org_recls_Entry_isReadOnly
  (JNIEnv *henv, jobject obj)
{
	return GetEntryBooleanProperty(henv, obj, Recls_IsFileReadOnly);
}

/*
 * Class:     org_recls_Entry
 * Method:    isDirectory
 * Signature: ()Z
 */
JNIEXPORT jboolean JNICALL Java_org_recls_Entry_isDirectory
  (JNIEnv *henv, jobject obj)
{
	return GetEntryBooleanProperty(henv, obj, Recls_IsFileDirectory);
}

/*
 * Class:     org_recls_Entry
 * Method:    isLink
 * Signature: ()Z
 */
JNIEXPORT jboolean JNICALL Java_org_recls_Entry_isLink
  (JNIEnv *henv, jobject obj)
{
	return GetEntryBooleanProperty(henv, obj, Recls_IsFileLink);
}

/*
 * Class:     org_recls_Entry
 * Method:    isUNC
 * Signature: ()Z
 */
JNIEXPORT jboolean JNICALL Java_org_recls_Entry_isUNC
  (JNIEnv *henv, jobject obj)
{
	return GetEntryBooleanProperty(henv, obj, Recls_IsFileUNC);
}

/*
 * Class:     org_recls_Entry
 * Method:    getCreationTime
 * Signature: ()Ljava/util/Date;
 */
JNIEXPORT jlong JNICALL Java_org_recls_Entry_getCreationTime_1
  (JNIEnv *henv, jobject obj)
{
	return GetEntryTimeProperty(henv, obj, Recls_GetCreationTime);
}

/*
 * Class:     org_recls_Entry
 * Method:    getModificationTime
 * Signature: ()Ljava/util/Date;
 */
JNIEXPORT jlong JNICALL Java_org_recls_Entry_getModificationTime_1
  (JNIEnv *henv, jobject obj)
{
	return GetEntryTimeProperty(henv, obj, Recls_GetModificationTime);
}

/*
 * Class:     org_recls_Entry
 * Method:    getLastAccessTime
 * Signature: ()Ljava/util/Date;
 */
JNIEXPORT jlong JNICALL Java_org_recls_Entry_getLastAccessTime_1
  (JNIEnv *henv, jobject obj)
{
	return GetEntryTimeProperty(henv, obj, Recls_GetLastAccessTime);
}

/*
 * Class:     org_recls_Entry
 * Method:    getStatusChangeTime
 * Signature: ()Ljava/util/Date;
 */
JNIEXPORT jlong JNICALL Java_org_recls_Entry_getLastStatusChangeTime_1
  (JNIEnv *henv, jobject obj)
{
	return GetEntryTimeProperty(henv, obj, Recls_GetLastStatusChangeTime);
}

/* ////////////////////////////////////////////////////////////////////////// */

jfieldID GetEntryFieldID(JNIEnv *henv, jobject obj)
{
	jclass		clsEntry	=	henv->GetObjectClass(obj);
	jfieldID	fid			=	henv->GetFieldID(clsEntry, FIELDNAME_HENTRY, "I");

//	henv->DeleteLocalRef(clsEntry);

	return fid;
}

recls_info_t EntryFromJObject(JNIEnv *henv, jobject obj)
{
	jfieldID	fid = GetEntryFieldID(henv, obj);

	return reinterpret_cast<recls_info_t>(henv->GetIntField(obj, fid));
}

jstring GetEntryStringProperty(JNIEnv *henv, jobject obj, size_t (RECLS_CALLCONV_DEFAULT *pfn)(recls_info_t, recls_char_t*, size_t))
{
	recls_info_t	hEntry	=	EntryFromJObject(henv, obj);

	if(NULL == hEntry)
	{
		ThrowException(henv, CLASSNAME_ILLEGALSTATEX, "Entry is closed");

		return NULL;
	}
	else
	{
		file_path_buffer	buffer;
		size_t				cch = pfn(hEntry, &buffer[0], buffer.size());
	
#ifdef RECLS_CHAR_TYPE_IS_CHAR
		return henv->NewStringUTF(buffer.c_str());
#elif defined(RECLS_CHAR_TYPE_IS_WCHAR
		return henv->NewString(buffer.c_str());
#else /* ?char type */
# error recls char type must be discriminated
#endif /* char type */
	} 
}

jlong GetEntryTimeProperty(JNIEnv *henv, jobject obj, recls_time_t (RECLS_CALLCONV_DEFAULT *pfn)(recls_info_t))
{
	recls_info_t	hEntry	=	EntryFromJObject(henv, obj);

	if(NULL == hEntry)
	{
		ThrowException(henv, CLASSNAME_ILLEGALSTATEX, "Entry is closed");

		return NULL;
	}
	else
	{
		recls_time_t	rt = pfn(hEntry);
		time_t			t;
		jlong			r;

#if defined(RECLS_PLATFORM_IS_WIN32)
		t = winstl::FILETIMEToUNIXTime(rt);
#elif defined(RECLS_PLATFORM_IS_UNIX)
		t = rt;
#else
# error Platform not (yet) recognised
#endif /* platform */

		r = (jlong)t * 1000;

		return r;
	} 
}

jboolean GetEntryBooleanProperty(JNIEnv *henv, jobject obj, recls_bool_t (RECLS_CALLCONV_DEFAULT *pfn)(recls_info_t))
{
	recls_info_t	hEntry	=	EntryFromJObject(henv, obj);

	if(NULL == hEntry)
	{
		ThrowException(henv, CLASSNAME_ILLEGALSTATEX, "Entry is closed");

		return NULL;
	}
	else
	{
		return pfn(hEntry);
	} 
}

