/* /////////////////////////////////////////////////////////////////////////////
 * File:        sample_directories_C.c
 *
 * Purpose:     Implementation file for the sample_directories_C project.
 *
 * Created:     20th September 2005
 * Updated:     5th July 2006
 *
 * Status:      Wizard-generated
 *
 * License:     Copyright (c) 2005-2006, Synesis Software Pty Ltd.
 *              All rights reserved.
 *
 *              (Licensed under the Synesis Software Open License)
 *
 *              This source code is placed into the public domain 2006
 *              by Synesis Software Pty Ltd. There are no restrictions
 *              whatsoever to your use of the software.
 *
 * ////////////////////////////////////////////////////////////////////////// */


/* recls Header Files */
#include <recls/recls.h>
#ifndef RECLS_NO_IMPLICIT_LINK
# include <recls/implicit_link.h>
#endif /* !RECLS_NO_IMPLICIT_LINK */

/* Standard C Header Files */
#include <stdio.h>
#include <stdlib.h> /* For malloc() */

/* /////////////////////////////////////////////////////////////////////////////
 * Helper functions
 */

/** Returns a string form of the given error
 *
 * \note This function returns a C-style string allocated from the heap, using
 *        malloc(). If this result is not part of pre-termination error
 *        display, the caller must release the memory using free(), otherwise
 *        a leak will ensue
 */
static char *Recls_strerror(recls_rc_t rc);
static char *Recls_strerror(recls_rc_t rc)
{
    size_t  cch     =   Recls_GetErrorString(rc, NULL, 0);
    char    *msg    =   (char*)malloc(1 + cch);

    if(NULL != msg)
    {
        Recls_GetErrorString(rc, msg, 1 + cch);
    }

    return msg;
}

/* ////////////////////////////////////////////////////////////////////////// */

int main()
{
    hrecls_t    hSrch;
    recls_rc_t  rc = Recls_Search(  "."                                     /* search from the current directory */
                                ,   Recls_GetWildcardsAll()                 /* match all patterns                */
                                ,   RECLS_F_DIRECTORIES | RECLS_F_RECURSIVE /* look recursively, for directories */
                                ,   &hSrch);

    if(RECLS_RC_NO_MORE_DATA == rc)
    {
        fprintf(stderr, "No subdirectories\n");
    }
    else if(RECLS_FAILED(rc))
    {
        fprintf(stderr, "Search failed: %s\n", Recls_strerror(rc));
    }
    else
    {
        recls_info_t    entry;
        int             n   =   0;

        fprintf(stdout, "Listing the sub-directories of the current directory:\n");

        do
        {
            Recls_GetDetails(hSrch, &entry);

            fprintf(stdout, "  %s\n", entry->path.begin);

            ++n;

        } while(RECLS_RC_OK == Recls_GetNext(hSrch));

        fprintf(stdout, "    %d directories\n", n);

        Recls_SearchClose(hSrch);
    }

    return 0;
}


/* ////////////////////////////////////////////////////////////////////////// */
