/* /////////////////////////////////////////////////////////////////////////////
 * File:        sample_directories_C.c
 *
 * Purpose:     Implementation file for the sample_directories_C project.
 *
 * Created:     20th September 2005
 * Updated:     27th December 2005
 *
 * Status:      Wizard-generated
 *
 * License:     (Licensed under the Synesis Software Open License)
 *
 *              Copyright (c) 2005, Synesis Software Pty Ltd.
 *              All rights reserved.
 *
 *              www:        http://www.synesis.com.au/software
 *
 *              This source code is placed into the public domain 2005
 *              by Synesis Software Pty Ltd. There are no restrictions
 *              whatsoever to your use of the software. 
 *
 *              This source code is provided by Synesis Software Pty Ltd "as is"
 *              and any warranties, whether expressed or implied, including, but
 *              not limited to, the implied warranties of merchantability and
 *              fitness for a particular purpose are disclaimed. In no event
 *              shall the Synesis Software Pty Ltd be liable for any direct,
 *              indirect, incidental, special, exemplary, or consequential
 *              damages (including, but not limited to, procurement of
 *              substitute goods or services; loss of use, data, or profits; or
 *              business interruption) however caused and on any theory of
 *              liability, whether in contract, strict liability, or tort
 *              (including negligence or otherwise) arising in any way out of
 *              the use of this software, even if advised of the possibility of
 *              such damage. 
 *
 *              Neither the name of Synesis Software Pty Ltd nor the names of
 *              any subdivisions, employees or agents of Synesis Software Pty
 *              Ltd, nor the names of any other contributors to this software
 *              may be used to endorse or promote products derived from this
 *              software without specific prior written permission. 
 *
 * ////////////////////////////////////////////////////////////////////////// */


/* recls Header Files */
#include <recls/recls.h>
#ifndef RECLS_NO_IMPLICIT_LINK
# include <recls/implicit_link.h>
#endif /* !RECLS_NO_IMPLICIT_LINK */

/* Standard C Header Files */
#include <stdio.h>
#include <stdlib.h> /* For malloc() */

/* /////////////////////////////////////////////////////////////////////////////
 * Helper functions
 */

/** Returns a string form of the given error
 *
 * \note This function returns a C-style string allocated from the heap, using
 *        malloc(). If this result is not part of pre-termination error
 *        display, the caller must release the memory using free(), otherwise
 *        a leak will ensue
 */
static char *Recls_strerror(recls_rc_t rc);
static char *Recls_strerror(recls_rc_t rc)
{
    size_t  cch     =   Recls_GetErrorString(rc, NULL, 0);
    char    *msg    =   (char*)malloc(1 + cch);

    if(NULL != msg)
    {
        Recls_GetErrorString(rc, msg, 1 + cch);
    }

    return msg;
}

/* ////////////////////////////////////////////////////////////////////////// */

int main()
{
    hrecls_t    hSrch;
    recls_rc_t  rc = Recls_Search(  "."                                     /* search from the current directory */
                                ,   Recls_GetWildcardsAll()                 /* match all patterns                */
                                ,   RECLS_F_DIRECTORIES | RECLS_F_RECURSIVE /* look recursively, for directories */
                                ,   &hSrch);

    if(RECLS_RC_NO_MORE_DATA == rc)
    {
        fprintf(stderr, "No subdirectories\n");
    }
    else if(RECLS_FAILED(rc))
    {
        fprintf(stderr, "Search failed: %s\n", Recls_strerror(rc));
    }
    else
    {
        recls_info_t    entry;
        int             n   =   0;

        fprintf(stdout, "Listing the sub-directories of the current directory:\n");

        do
        {
            Recls_GetDetails(hSrch, &entry);

            fprintf(stdout, "  %s\n", entry->path.begin);

            ++n;

        } while(RECLS_RC_OK == Recls_GetNext(hSrch));

        fprintf(stdout, "    %d directories\n", n);

        Recls_SearchClose(hSrch);
    }

    return 0;
}


/* ////////////////////////////////////////////////////////////////////////// */
