/* /////////////////////////////////////////////////////////////////////////////
 * File:        sample_directories_COM.cpp
 *
 * Purpose:     Implementation file for the sample_directories_COM project.
 *
 * Created:     20th September 2005
 * Updated:     2nd December 2006
 *
 * Status:      Wizard-generated
 *
 * License:     (Licensed under the Synesis Software Open License)
 *
 *              Copyright (c) 2005-2006, Synesis Software Pty Ltd.
 *              All rights reserved.
 *
 *              www:        http://www.synesis.com.au/software
 *
 *              This source code is placed into the public domain 2005
 *              by Synesis Software Pty Ltd. There are no restrictions
 *              whatsoever to your use of the software. 
 *
 *              This source code is provided by Synesis Software Pty Ltd "as is"
 *              and any warranties, whether expressed or implied, including, but
 *              not limited to, the implied warranties of merchantability and
 *              fitness for a particular purpose are disclaimed. In no event
 *              shall the Synesis Software Pty Ltd be liable for any direct,
 *              indirect, incidental, special, exemplary, or consequential
 *              damages (including, but not limited to, procurement of
 *              substitute goods or services; loss of use, data, or profits; or
 *              business interruption) however caused and on any theory of
 *              liability, whether in contract, strict liability, or tort
 *              (including negligence or otherwise) arising in any way out of
 *              the use of this software, even if advised of the possibility of
 *              such damage. 
 *
 *              Neither the name of Synesis Software Pty Ltd nor the names of
 *              any subdivisions, employees or agents of Synesis Software Pty
 *              Ltd, nor the names of any other contributors to this software
 *              may be used to endorse or promote products derived from this
 *              software without specific prior written permission. 
 *
 * ////////////////////////////////////////////////////////////////////////// */


/* STLSoft Header Files */
#include <stlsoft/stlsoft.h>    /* If the compiler cannot find this, you are not using STLSoft 1.9.1 or later, as required. */
#include <stlsoft/smartptr/ref_ptr.hpp>

#include <comstl/comstl.h>
#include <comstl/collections/collection_sequence.hpp>
#include <comstl/util/creation_functions.hpp>
#include <comstl/collections/enumeration_policies.hpp>
#include <comstl/util/initialisers.hpp>
#include <comstl/interface/interface_cast.hpp>
#include <comstl/conversion/method_cast.hpp>
#include <comstl/util/value_policies.hpp>

#include <comstl/error/errorinfo_desc.hpp>

#include <winstl/winstl.h>
#include <winstl/error/error_desc.hpp>

/* Standard C++ Header Files */
#include <exception>
#include <iostream>

using std::cerr;
using std::cin;
using std::cout;
using std::endl;

#ifdef __WATCOMC__
namespace std
{
    using ::exception;
}
#endif /* __WATCOMC__ */


/* Standard C Header Files */
#include <stdio.h>

#include "../../mappings/COM/recls.com.h"
#include "../../mappings/COM/recls.com_i.c"

/* ////////////////////////////////////////////////////////////////////////// */

static int main_(int /* argc */, char ** /*argv*/)
{
    using namespace comstl;

    IFileSearch *fileSearch;
    HRESULT     hr  =   co_create_instance(CLSID_FileSearch, &fileSearch);

    if(FAILED(hr))
    {
        cout << "Failed to create recls server object: " << winstl::error_desc(hr).c_str() << endl;
    }
    else
    {
		stlsoft::ref_ptr<IFileSearch>	fs(fileSearch, false); // false to eat the reference
        IUnknown						*punkSrchColl;

        hr  =   fs->Search( L"."
                        ,   L"*.*"
                        ,   RECLS_F_DIRECTORIES | RECLS_F_RECURSIVE | RECLS_F_MARK_DIRS
                        ,   &punkSrchColl);

        if(FAILED(hr))
        {
#if 0
            cout << "Search failed: " << winstl::error_desc(hr).c_str() << endl;
#else /* ? 0 */
            cout << "Search failed: " << errorinfo_desc().c_str() << endl;
#endif /* 0 */
        }
        else
        {
            typedef collection_sequence<ISearchCollection
                                    ,   IEnumFileEntry
                                    ,   IFileEntry*
                                    ,   interface_policy<IFileEntry>
                                    >       coll_t;

            stlsoft::ref_ptr<IUnknown>	u_(punkSrchColl, false);
            coll_t				        coll(interface_cast_addref<ISearchCollection*>(punkSrchColl), true);
            int						    n = 0;

            { for(coll_t::iterator b = coll.begin(); b != coll.end(); ++b, ++n)
            {
                BSTR    bstrPath    =   method_cast<BSTR>(*b, &IFileEntry::get_Path);

                if(NULL != bstrPath)
                {
                    fwprintf(stderr, L"  %s\n", bstrPath);

                    ::SysFreeString(bstrPath);
                }
            }}

            cout << "    " << n << " directories" << endl;
        }
    }

    return 0;
}

int main(int argc, char *argv[])
{
#if 0
    { for(size_t i = 0; i < 0xffffffff; ++i){} }
#endif /* 0 */

    try
    {
        comstl::com_initialiser coinit;

        return main_(argc, argv);
    }
    catch(std::exception &x)
    {
        fprintf(stderr, "Unhandled error: %s\n", x.what());
    }
    catch(...)
    {
        fprintf(stderr, "Unhandled unknown error\n");
    }

    return EXIT_FAILURE;
}

/* ////////////////////////////////////////////////////////////////////////// */
