/* /////////////////////////////////////////////////////////////////////////////
 * File:        recls_fileinfo_win32.cpp
 *
 * Purpose:     Win32 implementation for the file information blocks of the recls API.
 *
 * Created:     16th August 2003
 * Updated:     26th February 2007
 *
 * Home:        http://recls.org/
 *
 * Copyright (c) 2003-2007, Matthew Wilson and Synesis Software
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 *
 * - Redistributions of source code must retain the above copyright notice, this
 *   list of conditions and the following disclaimer. 
 * - Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 * - Neither the name(s) of Matthew Wilson and Synesis Software nor the names of
 *   any contributors may be used to endorse or promote products derived from
 *   this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * ////////////////////////////////////////////////////////////////////////// */


/* /////////////////////////////////////////////////////////////////////////////
 * Includes
 */

#include <recls/recls.h>
#include <recls/assert.h>
#include "recls_impl.h"

#include <stlsoft/stlsoft.h>    /* If the compiler cannot find this, you are not using STLSoft 1.9.1 or later, as required. */
#if _STLSOFT_VER < 0x0109012a
# error STLSoft version 1.9.1 (beta 42) or greater required. (www.stlsoft.org/downloads.html)
#endif /* STLSoft version */

#if defined(_RECLS_MT)
# include <winstl/winstl.h>
# include <winstl/synch/atomic_functions.h>
#endif /* _RECLS_MT */

#include "recls_debug.h"

#include "recls_atomic.h"

/* /////////////////////////////////////////////////////////////////////////////
 * Namespace
 */

#if !defined(RECLS_NO_NAMESPACE)
namespace recls
{
#endif /* !RECLS_NO_NAMESPACE */

#if defined(_RECLS_MT)
winstl_ns_using(ws_sint32_t)
winstl_ns_using(atomic_increment)
winstl_ns_using(atomic_predecrement)
winstl_ns_using(atomic_read)
#endif /* _RECLS_MT */

/* /////////////////////////////////////////////////////////////////////////////
 * Functions
 */

RECLS_FNDECL(void) RC_Increment(rc_atomic_t volatile *p)
{
#if defined(_RECLS_MT)
    atomic_increment(static_cast<ws_sint32_t volatile*>(static_cast<void volatile*>(p)));
#else /* ? _RECLS_MT */
    ++*p;
#endif /* _RECLS_MT */
}

RECLS_FNDECL(rc_atomic_t) RC_PreDecrement(rc_atomic_t volatile *p)
{
#if defined(_RECLS_MT)
    return atomic_predecrement(static_cast<ws_sint32_t volatile*>(static_cast<void volatile*>(p)));
#else /* ? _RECLS_MT */
    return --*p;
#endif /* _RECLS_MT */
}

RECLS_FNDECL(rc_atomic_t) RC_ReadValue(rc_atomic_t volatile *p)
{
#if defined(_RECLS_MT)
    return atomic_read(static_cast<ws_sint32_t volatile*>(static_cast<void volatile*>(p)));
#else /* ? _RECLS_MT */
    return *p;
#endif /* _RECLS_MT */
}

/* /////////////////////////////////////////////////////////////////////////////
 * Namespace
 */

#if !defined(RECLS_NO_NAMESPACE)
} /* namespace recls */
#endif /* !RECLS_NO_NAMESPACE */

/* ////////////////////////////////////////////////////////////////////////// */
