/* /////////////////////////////////////////////////////////////////////////////
 * File:        recls_impl.h
 *
 * Purpose:     Implementation header.
 *
 * Created:     7th March 2005
 * Updated:     27th February 2007
 *
 * Home:        http://recls.org/
 *
 * Copyright (c) 2005-2007, Matthew Wilson and Synesis Software
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted in accordance with the license and warranty
 * information described in recls.h (included in this distribution, or available
 * from http://recls.org/)
 *
 * ////////////////////////////////////////////////////////////////////////// */


#ifndef RECLS_INCL_H_RECLS_IMPL
#define RECLS_INCL_H_RECLS_IMPL

/* File version */
#ifndef RECLS_DOCUMENTATION_SKIP_SECTION
# define RECLS_VER_H_RECLS_IMPL_MAJOR       3
# define RECLS_VER_H_RECLS_IMPL_MINOR       2
# define RECLS_VER_H_RECLS_IMPL_REVISION    4
# define RECLS_VER_H_RECLS_IMPL_EDIT        31
#endif /* !RECLS_DOCUMENTATION_SKIP_SECTION */

/** \file recls_impl.h Implementation header.
 */

/* /////////////////////////////////////////////////////////////////////////////
 * Includes
 */

#include <recls/recls.h>

#ifdef RECLS_PURE_API
# error Cannot define RECLS_PURE_API for the implementation of recls
#endif /* RECLS_PURE_API */

#if defined(RECLS_PLATFORM_IS_UNIX)
# if defined(WIN32) && \
     defined(EMULATE_UNIX_ON_WIN32)
#  include <windows.h>
# endif /* WIN32 && EMULATE_UNIX_ON_WIN32 */
#endif /* RECLS_PLATFORM_IS_UNIX */

/* /////////////////////////////////////////////////////////////////////////////
 * C++ stuff
 */

#ifdef __cplusplus
# include <stlsoft/stlsoft.h>    /* If the compiler cannot find this, you are not using STLSoft 1.9.1 or later, as required. */
# if _STLSOFT_VER < 0x0109012a
#  error STLSoft version 1.9.1 (beta 42) or greater required. (www.stlsoft.org/downloads.html)
# endif /* STLSoft version */
# include <stlsoft/conversion/sap_cast.hpp>
#endif /* __cplusplus */

/* /////////////////////////////////////////////////////////////////////////////
 * Exceptions
 */

#ifdef __cplusplus
# if defined(__STLSOFT_CF_EXCEPTION_SUPPORT) || \
     defined(STLSOFT_CF_EXCEPTION_SUPPORT)
#  define _RECLS_EXCEPTION_SUPPORT
#  ifdef __STLSOFT_CF_THROW_BAD_ALLOC
#   define RECLS_COMPILER_THROWS_ON_NEW_FAIL
#  endif /* __STLSOFT_CF_THROW_BAD_ALLOC */
# endif /* __STLSOFT_CF_EXCEPTION_SUPPORT || STLSOFT_CF_EXCEPTION_SUPPORT */
#endif /* __cplusplus */

/* /////////////////////////////////////////////////////////////////////////////
 * Debugging?
 */

/** \def _RECLS_DEBUG
 *
 * \brief Used to discriminate intrinsic debug level.
 *
 * Defined if NDEBUG is not defined on UNIX, or either DEBUG or _DEBUG are
 * defined on Win32.
 */

#if !defined(NDEBUG)
# if defined(RECLS_PLATFORM_IS_UNIX)
#  define _RECLS_DEBUG
# elif defined(RECLS_PLATFORM_IS_WIN32) && \
       (   defined(DEBUG) || \
           defined(_DEBUG))
#  define _RECLS_DEBUG
# endif /* platform */
#endif /* debug */

/** \def RECLS_DEBUG_LEVEL
 *
 * \brief Debugging level (0 suppresses debug functionality).
 *
 * Level 1 includes assertions
 * Level 2 includes tracing.
 */

#ifdef _RECLS_DEBUG
# ifndef RECLS_DEBUG_LEVEL
#  define RECLS_DEBUG_LEVEL 1
# endif /* RECLS_DEBUG_LEVEL */
#else /* ? _RECLS_DEBUG */
# ifdef RECLS_DEBUG_LEVEL
#  undef RECLS_DEBUG_LEVEL
# endif /* RECLS_DEBUG_LEVEL */
# define RECLS_DEBUG_LEVEL  0
#endif /* _RECLS_DEBUG */

/* /////////////////////////////////////////////////////////////////////////////
 * Contract enforcement
 */

/** \def RECLS_ENFORCING_CONTRACTS If defined, it indicates that contract enforcement checking will happen */

#if defined(_RECLS_DEBUG) || \
    defined(RECLS_ENFORCE_CONTRACTS)
# define RECLS_ENFORCING_CONTRACTS
#endif /* _RECLS_DEBUG || RECLS_ENFORCE_CONTRACTS */

/* /////////////////////////////////////////////////////////////////////////////
 * Multithreading
 */

/** \def _RECLS_MT If defined, it indicates a multithreaded build */

#if defined(RECLS_PLATFORM_IS_UNIX)
# if defined(_REENTRANT)
#  define _RECLS_MT
# elif defined(WIN32) && \
       defined(EMULATE_UNIX_ON_WIN32) && \
       (    defined(_MT) || \
            defined(__MT__))
#  define _RECLS_MT
#  ifndef _REENTRANT
#   define _REENTRANT
#  endif /* !_REENTRANT */
# endif /* threads? */
#elif defined(RECLS_PLATFORM_IS_WIN32)
# if defined(_MT) || \
     defined(__MT__)
#  define _RECLS_MT
# endif /* threads? */
#endif /* platform */

/* /////////////////////////////////////////////////////////////////////////////
 * RECLS_NUM_ELEMENTS
 */

#ifndef RECLS_NUM_ELEMENTS
# if defined(stlsoft_num_elements)
#  define RECLS_NUM_ELEMENTS(x)             stlsoft_num_elements(x)
# else /* ? stlsoft_num_elements */
#  define RECLS_NUM_ELEMENTS(x)             (sizeof(x) / sizeof((x)[0]))
# endif /* stlsoft_num_elements */
#endif /* !RECLS_NUM_ELEMENTS */

/* /////////////////////////////////////////////////////////////////////////////
 * Multi-part patterns
 */

#if (   !defined(RECLS_COMPILER_IS_MSVC) || \
        _MSC_VER >= 1200) && \
    !defined(RECLS_COMPILER_IS_WATCOM)
# define _RECLS_SUPPORTS_MULTIPATTERN
#endif /* compiler */

/* ////////////////////////////////////////////////////////////////////////// */

#endif /* !RECLS_INCL_H_RECLS_IMPL */

/* ////////////////////////////////////////////////////////////////////////// */
