/* /////////////////////////////////////////////////////////////////////////////
 * File:        test_C_1.c
 *
 * Purpose:     Implementation file for the test_C_1 project.
 *
 * Created:     7th October 2005
 * Updated:     24th December 2005
 *
 * Status:      Wizard-generated
 *
 * License:     (Licensed under the Synesis Software Open License)
 *
 *              Copyright 1999-2005, Synesis Software Pty Ltd.
 *              All rights reserved.
 *
 *              www:        http://www.synesis.com.au/software
 *
 *              This source code is placed into the public domain 2005
 *              by Synesis Software Pty Ltd. There are no restrictions
 *              whatsoever to your use of the software. 
 *
 *              This source code is provided by Synesis Software Pty Ltd "as is"
 *              and any warranties, whether expressed or implied, including, but
 *              not limited to, the implied warranties of merchantability and
 *              fitness for a particular purpose are disclaimed. In no event
 *              shall the Synesis Software Pty Ltd be liable for any direct,
 *              indirect, incidental, special, exemplary, or consequential
 *              damages (including, but not limited to, procurement of
 *              substitute goods or services; loss of use, data, or profits; or
 *              business interruption) however caused and on any theory of
 *              liability, whether in contract, strict liability, or tort
 *              (including negligence or otherwise) arising in any way out of
 *              the use of this software, even if advised of the possibility of
 *              such damage. 
 *
 *              Neither the name of Synesis Software Pty Ltd nor the names of
 *              any subdivisions, employees or agents of Synesis Software Pty
 *              Ltd, nor the names of any other contributors to this software
 *              may be used to endorse or promote products derived from this
 *              software without specific prior written permission. 
 *
 * ////////////////////////////////////////////////////////////////////////// */

/* STLSoft Header Files */
#include <stlsoft/stlsoft.h>    /* If the compiler cannot find this, you are not using STLSoft 1.9.1 or later, as required. */

/* PlatformSTL Header Files */
#include <platformstl/platformstl.h>
#include <platformstl/atomic_functions.h>

/* recls Header Files */
#include <recls/recls.h>

/* Standard C Header Files */
#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/stat.h>

/* /////////////////////////////////////////////////////////////////////////////
 * Forward declarations
 */

static int run_tests(void);

/* Test functions */
static int current_directory_files(int bRecurse, int bCheckDirectoryParts);
static int current_directory_directories(int bRecurse, int bCheckDirectoryParts);
static int root_directory_files(int bRecurse, int bCheckDirectoryParts);
static int root_directory_directories(int bRecurse, int bCheckDirectoryParts);

static int stat_current_directory(int bTrailingSlash);
static int stat_root_directory(int bTrailingSlash);

/* Helper functions */
static recls_uint32_t   calc_flags(recls_uint32_t flags, int bRecurse, int bCheckDirectoryParts);
static int              print_error(char const *message, recls_rc_t rc, recls_uint32_t flags);
static recls_rc_t       check_error(char const *message, recls_rc_t rc, ...);
static recls_rc_t       vcheck_error(char const *message, recls_rc_t rc, va_list args);
static recls_rc_t       check_entry(recls_info_t info, recls_uint32_t flags);

/* ////////////////////////////////////////////////////////////////////////// */

int main(int argc, char *argv[])
{
#if 0
    { size_t i; for(i = 0; i < 0xffffffff; ++i){} }
#endif /* 0 */

#if defined(_DEBUG) || \
    defined(__SYNSOFT_DBS_DEBUG)
    puts("test_C_1: " __STLSOFT_COMPILER_LABEL_STRING);
#endif /* debug */

    ((void)argc);
    ((void)argv);

    return run_tests();
}

/* ////////////////////////////////////////////////////////////////////////// */

static int run_tests(void)
{
    int (*search_tests[])(int, int) = 
    {
            current_directory_files
        ,   current_directory_directories
        ,   root_directory_files
        ,   root_directory_directories
    };

    int (*stat_tests[])(int) = 
    {
            stat_current_directory
        ,   stat_root_directory
    };

    int     res =   EXIT_SUCCESS;
    size_t  i;

    for(i = 0; i < STLSOFT_NUM_ELEMENTS(search_tests); ++i)
    {
        int perm;

        for(perm = 0; perm < 4; ++perm)
        {
            if(EXIT_SUCCESS != (*search_tests[i])(perm & 0x01, (perm >> 1) & 0x01))
            {
                res = EXIT_FAILURE;
            }
        }
    }

    return res;
}

/* ////////////////////////////////////////////////////////////////////////// */

static recls_uint32_t calc_flags(recls_uint32_t flags, int bRecurse, int bCheckDirectoryParts)
{
    if(bRecurse)
    {
        flags |= RECLS_F_RECURSIVE;
    }

    if(bRecurse)
    {
        flags |= RECLS_F_DIRECTORY_PARTS;
    }

    return flags;
}

static int print_error(char const *message, recls_rc_t rc, recls_uint32_t flags)
{
    recls_char_t    buff[1001];

    Recls_GetErrorString(rc, buff, STLSOFT_NUM_ELEMENTS(buff));

    fprintf(stderr, "%s: %s; flags = 0x%08x\n", message, buff, flags);

    return EXIT_SUCCESS;
}

static recls_rc_t check_entry(recls_info_t info, recls_uint32_t flags)
{
    if(NULL == info)
    {
        return check_error("Entry handle is null", RECLS_RC_UNEXPECTED);
    }
    else
    {
        const size_t    pathLen =   strlen(info->path.begin);
        struct stat     st;

        if('\0' != *info->path.end)
        {
            return check_error("path.end is not null", RECLS_RC_UNEXPECTED);
        }
        if(pathLen != (size_t)(info->path.end - info->path.begin))
        {
            return check_error("path length is invalid", RECLS_RC_UNEXPECTED);
        }

        /* Note: Cannot report error if fail to stat(), since the entry 
         * may have been removed
         */
        if(0 == stat(info->path.begin, &st))
        {
            const int           bIsDir_Stat     =   S_IFDIR == (st.st_mode & S_IFDIR);
            const int           bIsDir_recls    =   0 != Recls_IsFileDirectory(info);
            recls_filesize_t    size_recls;

            if(bIsDir_Stat != bIsDir_recls)
            {
                return check_error("stat() and recls do not agree on whether \"%s\" is a directory; This may not be an error: re-run the test", RECLS_RC_UNEXPECTED, info->path.begin);
            }

            Recls_GetSizeProperty(info, &size_recls);

#ifdef RECLS_PLATFORM_IS_WIN32
            if(st.st_size != size_recls.QuadPart)
#else /* ? platform */
            if(st.st_size != size_recls)
#endif /* platform */
            {
                return check_error("stat() and recls do not agree on size of \"%s\"; This may not be an error: re-run the test", RECLS_RC_UNEXPECTED, info->path.begin);
            }
        }

//      if(0 != strncmp(info->path.begin, info->d

        return RECLS_RC_OK;
    }
}

static recls_rc_t check_error(char const *message, recls_rc_t rc, ...)
{
    if(NULL != message)
    {
        va_list args;

        va_start(args, rc);

        rc = vcheck_error(message, rc, args);

        va_end(args);
    }

    return rc;
}

static recls_rc_t vcheck_error(char const *message, recls_rc_t rc, va_list args)
{
    stlsoft_assert(NULL != message);

    vfprintf(stderr, message, args);

    return rc;
}

/* ////////////////////////////////////////////////////////////////////////// */

static int current_directory_files(int bRecurse, int bCheckDirectoryParts)
{
    recls_uint32_t  flags   =   calc_flags(RECLS_F_FILES, bRecurse, bCheckDirectoryParts);
    hrecls_t        hSrch;
    recls_rc_t      rc      =   Recls_Search(".", Recls_GetWildcardsAll(), flags, &hSrch);

    if(RECLS_FAILED(rc))
    {
        return print_error("Failed to open search on current directory", rc, flags);
    }
    else
    {
        Recls_SearchClose(hSrch);

        return EXIT_SUCCESS;
    }
}

static int current_directory_directories(int bRecurse, int bCheckDirectoryParts)
{
    recls_uint32_t  flags   =   calc_flags(RECLS_F_DIRECTORIES, bRecurse, bCheckDirectoryParts);
    hrecls_t        hSrch;
    recls_rc_t      rc      =   Recls_Search(".", Recls_GetWildcardsAll(), flags, &hSrch);

    if(RECLS_FAILED(rc))
    {
        return print_error("Failed to open search on current directory", rc, flags);
    }
    else
    {
        recls_info_t    info;

        rc = Recls_GetDetails(hSrch, &info);

        for(; RECLS_SUCCEEDED(rc); )
        {
            rc = check_entry(info, flags);

            Recls_CloseDetails(info);

            if(RECLS_SUCCEEDED(rc))
            {
                rc = Recls_GetNextDetails(hSrch, &info);
            }
        }

        Recls_SearchClose(hSrch);

        if(rc != RECLS_RC_NO_MORE_DATA)
        {
            return print_error("Search terminated prematurely", rc, flags);
        }

        return EXIT_SUCCESS;
    }
}

static int root_directory_files(int bRecurse, int bCheckDirectoryParts)
{
    recls_uint32_t  flags   =   calc_flags(RECLS_F_FILES, bRecurse, bCheckDirectoryParts);
    hrecls_t        hSrch;
    recls_rc_t      rc      =   Recls_Search("/", Recls_GetWildcardsAll(), flags, &hSrch);

    if(RECLS_FAILED(rc))
    {
        return print_error("Failed to open search on root directory", rc, flags);
    }
    else
    {
        recls_info_t    info;

        rc = Recls_GetDetails(hSrch, &info);

        for(; RECLS_SUCCEEDED(rc); )
        {
            rc = check_entry(info, flags);

            Recls_CloseDetails(info);

            if(RECLS_SUCCEEDED(rc))
            {
                rc = Recls_GetNextDetails(hSrch, &info);
            }
        }

        Recls_SearchClose(hSrch);

        if(rc != RECLS_RC_NO_MORE_DATA)
        {
            return print_error("Search terminated prematurely", rc, flags);
        }

        return EXIT_SUCCESS;
    }
}

static int root_directory_directories(int bRecurse, int bCheckDirectoryParts)
{
    recls_uint32_t  flags   =   calc_flags(RECLS_F_DIRECTORIES, bRecurse, bCheckDirectoryParts);
    hrecls_t        hSrch;
    recls_rc_t      rc      =   Recls_Search("/", Recls_GetWildcardsAll(), flags, &hSrch);

    if(RECLS_FAILED(rc))
    {
        return print_error("Failed to open search on root directory", rc, flags);
    }
    else
    {
        Recls_SearchClose(hSrch);

        return EXIT_SUCCESS;
    }
}

/* ////////////////////////////////////////////////////////////////////////// */

static int stat_current_directory(int bTrailingSlash)
{
    char const      *dir    =   bTrailingSlash ? "./" : ".";
    recls_uint32_t  flags   =   ~(recls_uint32_t)(0);
    recls_info_t    info;
    recls_rc_t      rc      =   Recls_Stat("/", flags, &info);

    if(RECLS_FAILED(rc))
    {
        return print_error("Failed to stat() current directory", rc, flags);
    }
    else
    {
        Recls_CloseDetails(info);

        return EXIT_SUCCESS;
    }
}

static int stat_root_directory(int bTrailingSlash)
{

        return EXIT_SUCCESS;
}

/* ////////////////////////////////////////////////////////////////////////// */
